@@ -56,7 +56,7 @@ func init() {
 			return nil
 		},
 		Run: func(cmd *cobra.Command, args []string) {
-			_, err := Client.RemoveJob(azcopy.RemoveJobOptions{JobID: commandLineInput.JobID})
+			_, err := Client.RemoveJob(commandLineInput.JobID, azcopy.RemoveJobOptions{})
 			if err == nil {
 				glcm.Exit(func(format OutputFormat) string {
 					return fmt.Sprintf("Successfully removed log and job plan files for job %s.", commandLineInput.JobID)
