@@ -33,6 +33,7 @@ import (
 	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
 	"github.com/Azure/azure-sdk-for-go/sdk/storage/azblob/blob"
 	"github.com/Azure/azure-sdk-for-go/sdk/storage/azblob/lease"
+	"github.com/Azure/azure-storage-azcopy/v10/azcopy"
 	traverser2 "github.com/Azure/azure-storage-azcopy/v10/traverser"
 
 	"github.com/spf13/cobra"
@@ -234,7 +235,7 @@ func (cooked cookedListCmdArgs) handleListContainerCommand() (err error) {
 		return fmt.Errorf("failed to resolve target: %w", err)
 	}
 
-	level, err := DetermineLocationLevel(source.Value, cooked.location, true)
+	level, err := azcopy.DetermineLocationLevel(source.Value, cooked.location, true)
 	if err != nil {
 		return err
 	}
@@ -253,9 +254,30 @@ func (cooked cookedListCmdArgs) handleListContainerCommand() (err error) {
 
 	// check if user wants to get version id
 	getVersionId := containsProperty(cooked.properties, VersionId)
+	var reauthTok *common.ScopedAuthenticator
+	if at, ok := credentialInfo.OAuthTokenInfo.TokenCredential.(common.AuthenticateToken); ok { // We don't need two different tokens here since it gets passed in just the same either way.
+		// This will cause a reauth with StorageScope, which is fine, that's the original Authenticate call as it stands.
+		reauthTok = (*common.ScopedAuthenticator)(common.NewScopedCredential(at, common.ECredentialType.OAuthToken()))
+	}
+
+	options := traverser2.CreateClientOptions(common.AzcopyCurrentJobLogger, nil, reauthTok)
+	var fileClientOptions any
+	if cooked.location.IsFile() {
+		fileClientOptions = &common.FileClientOptions{AllowTrailingDot: cooked.trailingDot.IsEnabled()}
+	}
+
+	targetServiceClient, _ := common.GetServiceClientForLocation(
+		cooked.location,
+		source,
+		credentialInfo.CredentialType,
+		credentialInfo.OAuthTokenInfo.TokenCredential,
+		&options,
+		fileClientOptions,
+	)
 
 	traverser, err := traverser2.InitResourceTraverser(source, cooked.location, ctx, traverser2.InitResourceTraverserOptions{
-		Credential: &credentialInfo,
+		Client:         targetServiceClient,
+		CredentialType: credentialInfo.CredentialType,
 
 		TrailingDotOption: cooked.trailingDot,
 
@@ -374,7 +396,7 @@ func (l AzCopyListObject) String() string {
 	return l.StringEncoding
 }
 
-func (cooked cookedListCmdArgs) newListObject(object traverser2.StoredObject, level LocationLevel) AzCopyListObject {
+func (cooked cookedListCmdArgs) newListObject(object traverser2.StoredObject, level azcopy.LocationLevel) AzCopyListObject {
 	path := getPath(object.ContainerName, object.RelativePath, level, object.EntityType)
 	contentLength := sizeToString(object.Size, cooked.MachineReadable)
 
@@ -491,7 +513,7 @@ func ByteSizeToString(size int64) string {
 	return strconv.FormatFloat(floatSize, 'f', 2, 64) + " " + units[unit]
 }
 
-func getPath(containerName, relativePath string, level LocationLevel, entityType common.EntityType) string {
+func getPath(containerName, relativePath string, level azcopy.LocationLevel, entityType common.EntityType) string {
 	builder := strings.Builder{}
 	if level == level.Service() {
 		builder.WriteString(containerName + "/")
