@@ -21,13 +21,11 @@
 package cmd
 
 import (
-	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 
+	"github.com/Azure/azure-storage-azcopy/v10/azcopy"
 	"github.com/Azure/azure-storage-azcopy/v10/common"
-	"github.com/Azure/azure-storage-azcopy/v10/ste"
 	"github.com/spf13/cobra"
 )
 
@@ -37,35 +35,6 @@ type LoginStatusOptions struct {
 	Method      bool
 }
 
-type LoginStatus struct {
-	Valid       bool
-	TenantID    string
-	AADEndpoint string
-	AuthMethod  common.AutoLoginType
-}
-
-func (options LoginStatusOptions) process() (LoginStatus, error) {
-	// getting current token info and refreshing it with GetTokenInfo()
-	ctx := context.WithValue(context.TODO(), ste.ServiceAPIVersionOverride, ste.DefaultServiceApiVersion)
-	uotm := Client.GetUserOAuthTokenManagerInstance()
-	tokenInfo, err := uotm.GetTokenInfo(ctx)
-	var status = LoginStatus{
-		Valid: err == nil && !tokenInfo.IsExpired(),
-	}
-	if status.Valid {
-		status.TenantID = tokenInfo.Tenant
-		status.AADEndpoint = tokenInfo.ActiveDirectoryEndpoint
-		status.AuthMethod = tokenInfo.LoginType
-		return status, nil
-	} else {
-		return status, errors.New("You are currently not logged in. Please login using 'azcopy login'")
-	}
-}
-
-func RunLoginStatus(options LoginStatusOptions) (LoginStatus, error) {
-	return options.process()
-}
-
 type LoginStatusOutput struct {
 	Valid       bool    `json:"valid"`
 	TenantID    *string `json:"tenantID,omitempty"`
@@ -92,7 +61,7 @@ func init() {
 					glcm.Info(fmt.Sprintf(format, a...))
 				}
 			}
-			status, _ := RunLoginStatus(commandLineInput)
+			status, _ := Client.GetLoginStatus(azcopy.GetLoginStatusOptions{})
 			var Info = LoginStatusOutput{
 				Valid: status.Valid,
 			}
@@ -110,8 +79,8 @@ func init() {
 				}
 
 				if commandLineInput.Method {
-					logText(fmt.Sprintf("Authorized using %s", status.AuthMethod))
-					method := status.AuthMethod.String()
+					logText(fmt.Sprintf("Authorized using %s", status.LoginType))
+					method := status.LoginType.String()
 					Info.AuthMethod = &method
 				}
 			} else {
